@@ -254,19 +254,8 @@ meta_get_option_context (void)
   bindtextdomain (GETTEXT_PACKAGE, MUTTER_LOCALEDIR);
   bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 
-  /* We must set the windowing backend here, because Clutter creates the backend
-     object when the first call is made.
-
-     We consider running from mutter-launch equivalent to running from bare metal.
-  */
-  if (getenv ("WESTON_LAUNCHER_SOCK"))
-    clutter_set_windowing_backend (CLUTTER_WINDOWING_EGL);
-
   ctx = g_option_context_new (NULL);
   g_option_context_add_main_entries (ctx, meta_options, GETTEXT_PACKAGE);
-  g_option_context_add_group (ctx, clutter_get_option_group_without_init ());
-  g_option_context_add_group (ctx, cogl_get_option_group ());
-
   return ctx;
 }
 
@@ -354,7 +343,8 @@ meta_select_display (char *display_arg)
   else
     display_name = g_getenv ("MUTTER_DISPLAY");
 
-  g_setenv ("DISPLAY", display_name, TRUE);
+  if (display_name)
+    g_setenv ("DISPLAY", display_name, TRUE);
 }
 
 static void
@@ -389,7 +379,8 @@ meta_init (void)
 {
   struct sigaction act;
   sigset_t empty_mask;
-  
+  ClutterSettings *clutter_settings;
+
   sigemptyset (&empty_mask);
   act.sa_handler = SIG_IGN;
   act.sa_mask    = empty_mask;
@@ -410,6 +401,10 @@ meta_init (void)
   if (g_getenv ("MUTTER_DEBUG"))
     meta_set_debugging (TRUE);
 
+  /* We consider running from mutter-launch equivalent to running from bare metal. */
+  if (getenv ("WESTON_LAUNCHER_SOCK"))
+    clutter_set_windowing_backend (CLUTTER_WINDOWING_EGL);
+
   meta_set_is_wayland_compositor (opt_wayland);
 
   if (g_get_home_dir ())
@@ -441,7 +436,7 @@ meta_init (void)
     meta_fatal ("Can't specify both SM save file and SM client id\n");
   
   meta_main_loop = g_main_loop_new (NULL, FALSE);
-  
+
   meta_ui_init ();
 
   /* If we are running with wayland then we don't wait until we have
@@ -455,6 +450,13 @@ meta_init (void)
        */
       meta_clutter_init ();
     }
+
+  /*
+   * XXX: We cannot handle high dpi scaling yet, so fix the scale to 1
+   * for now.
+   */
+  clutter_settings = clutter_settings_get_default ();
+  g_object_set (clutter_settings, "window-scaling-factor", 1, NULL);
 }
 
 /**
