@@ -115,11 +115,7 @@ surface_process_damage (MetaWaylandSurface *surface,
       cairo_rectangle_int_t rect;
       cairo_region_get_rectangle (region, i, &rect);
       meta_surface_actor_damage_area (surface->surface_actor,
-                                      rect.x,
-                                      rect.y,
-                                      rect.width,
-                                      rect.height,
-                                      NULL);
+                                      rect.x, rect.y, rect.width, rect.height);
     }
 }
 
@@ -277,8 +273,6 @@ ensure_buffer_texture (MetaWaylandBuffer *buffer)
     }
 
   buffer->texture = texture;
-  buffer->width = cogl_texture_get_width (texture);
-  buffer->height = cogl_texture_get_height (texture);
 }
 
 static void
@@ -328,11 +322,11 @@ toplevel_surface_commit (MetaWaylandSurface             *surface,
       /* We resize X based surfaces according to X events */
       if (buffer != NULL && window->client_type == META_WINDOW_CLIENT_TYPE_WAYLAND)
         {
-          int new_width;
-          int new_height;
+          int new_width, new_height;
+
+          new_width = cogl_texture_get_width (buffer->texture);
+          new_height = cogl_texture_get_height (buffer->texture);
 
-          new_width = surface->buffer->width;
-          new_height = surface->buffer->height;
           if (new_width != window->rect.width ||
               new_height != window->rect.height ||
               pending->dx != 0 ||
@@ -340,6 +334,25 @@ toplevel_surface_commit (MetaWaylandSurface             *surface,
             meta_window_move_resize_wayland (window, new_width, new_height, pending->dx, pending->dy);
         }
     }
+
+  if (pending->frame_extents_changed)
+    meta_window_set_custom_frame_extents (surface->window, &pending->frame_extents);
+
+  if (pending->maximized.changed)
+    {
+      if (pending->maximized.value)
+        meta_window_maximize (surface->window, META_MAXIMIZE_HORIZONTAL | META_MAXIMIZE_VERTICAL);
+      else
+        meta_window_unmaximize (surface->window, META_MAXIMIZE_HORIZONTAL | META_MAXIMIZE_VERTICAL);
+    }
+
+  if (pending->fullscreen.changed)
+    {
+      if (pending->fullscreen.value)
+        meta_window_make_fullscreen (surface->window);
+      else
+        meta_window_unmake_fullscreen (surface->window);
+    }
 }
 
 static void
@@ -363,6 +376,10 @@ double_buffered_state_init (MetaWaylandDoubleBufferedState *state)
   state->buffer_destroy_listener.notify =
     surface_handle_pending_buffer_destroy;
   wl_list_init (&state->frame_callback_list);
+
+  state->frame_extents_changed = FALSE;
+  state->maximized.changed = FALSE;
+  state->fullscreen.changed = FALSE;
 }
 
 static void
@@ -554,23 +571,6 @@ const struct wl_surface_interface meta_wayland_surface_interface = {
   meta_wayland_surface_set_buffer_scale
 };
 
-static void
-meta_wayland_surface_free (MetaWaylandSurface *surface)
-{
-  MetaWaylandCompositor *compositor = surface->compositor;
-
-  compositor->surfaces = g_list_remove (compositor->surfaces, surface);
-
-  surface_set_buffer (surface, NULL);
-  double_buffered_state_destroy (&surface->pending);
-  g_object_unref (surface->surface_actor);
-  if (surface->resource)
-    wl_resource_set_user_data (surface->resource, NULL);
-  g_slice_free (MetaWaylandSurface, surface);
-
-  meta_wayland_compositor_repick (compositor);
-}
-
 static void
 unparent_actor (MetaWaylandSurface *surface)
 {
@@ -580,46 +580,55 @@ unparent_actor (MetaWaylandSurface *surface)
   clutter_actor_remove_child (parent_actor, CLUTTER_ACTOR (surface->surface_actor));
 }
 
-static void
-destroy_window (MetaWaylandSurface *surface)
+void
+meta_wayland_surface_window_unmanaged (MetaWaylandSurface *surface)
 {
-  MetaDisplay *display = meta_get_display ();
-  guint32 timestamp = meta_display_get_current_time_roundtrip (display);
-
-  /* Remove our actor from the parent, so it doesn't get destroyed when
-   * the MetaWindowActor is destroyed. */
+  /* The window is being unmanaged. Unparent our surface actor
+   * before the window actor is destroyed, as we need to hold
+   * onto it... */
   unparent_actor (surface);
 
-  g_assert (surface->window != NULL);
-  meta_window_unmanage (surface->window, timestamp);
   surface->window = NULL;
 }
 
 static void
-meta_wayland_surface_resource_destroy_cb (struct wl_resource *resource)
+destroy_window (MetaWaylandSurface *surface)
+{
+  if (surface->window)
+    {
+      MetaDisplay *display = meta_get_display ();
+      guint32 timestamp = meta_display_get_current_time_roundtrip (display);
+
+      meta_window_unmanage (surface->window, timestamp);
+    }
+
+  g_assert (surface->window == NULL);
+}
+
+static void
+wl_surface_destructor (struct wl_resource *resource)
 {
   MetaWaylandSurface *surface = wl_resource_get_user_data (resource);
+  MetaWaylandCompositor *compositor = surface->compositor;
 
-  /* There are four cases here:
-     - An X11 unmanaged window -> surface is NULL, nothing to do
-     - An X11 unmanaged window, but we got the wayland event first ->
-       just clear the resource pointer
-     - A wayland surface without window (destroyed before set_toplevel) ->
-       need to free the surface itself
-     - A wayland window -> need to unmanage
-  */
+  /* If we still have a window at the time of destruction, that means that
+   * the client is disconnecting, as the resources are destroyed in a random
+   * order. Simply destroy the window in this case. */
+  if (surface->window)
+    destroy_window (surface);
 
-  if (surface)
-    {
-      surface->resource = NULL;
+  compositor->surfaces = g_list_remove (compositor->surfaces, surface);
 
-      /* NB: If the surface corresponds to an X window then we will be
-       * sure to free the MetaWindow according to some X event. */
-      if (surface->window && surface->window->client_type == META_WINDOW_CLIENT_TYPE_WAYLAND)
-        destroy_window (surface);
+  surface_set_buffer (surface, NULL);
+  double_buffered_state_destroy (&surface->pending);
 
-      meta_wayland_surface_free (surface);
-    }
+  g_object_unref (surface->surface_actor);
+
+  if (surface->resource)
+    wl_resource_set_user_data (surface->resource, NULL);
+  g_slice_free (MetaWaylandSurface, surface);
+
+  meta_wayland_compositor_repick (compositor);
 }
 
 MetaWaylandSurface *
@@ -633,14 +642,14 @@ meta_wayland_surface_create (MetaWaylandCompositor *compositor,
   surface->compositor = compositor;
 
   surface->resource = wl_resource_create (client, &wl_surface_interface, version, id);
-  wl_resource_set_implementation (surface->resource, &meta_wayland_surface_interface, surface,
-				  meta_wayland_surface_resource_destroy_cb);
+  wl_resource_set_implementation (surface->resource, &meta_wayland_surface_interface, surface, wl_surface_destructor);
+
+  surface->buffer_destroy_listener.notify = surface_handle_buffer_destroy;
+  surface->surface_actor = g_object_ref_sink (meta_surface_actor_new (surface));
+  clutter_actor_set_reactive (CLUTTER_ACTOR (surface->surface_actor), TRUE);
 
   double_buffered_state_init (&surface->pending);
 
-  surface->buffer_destroy_listener.notify = surface_handle_buffer_destroy;
-
-  surface->surface_actor = g_object_ref_sink (meta_surface_actor_new ());
   return surface;
 }
 
@@ -689,6 +698,25 @@ create_surface_extension (MetaWaylandSurfaceExtension *extension,
   return TRUE;
 }
 
+static void
+xdg_shell_use_unstable_version (struct wl_client *client,
+                                struct wl_resource *resource,
+                                int32_t version)
+{
+  if (version != XDG_SHELL_VERSION_CURRENT)
+    g_warning ("Bad xdg_shell version: %d", version);
+}
+
+static void
+xdg_shell_pong (struct wl_client *client,
+                struct wl_resource *resource,
+                uint32_t serial)
+{
+  MetaDisplay *display = meta_get_display ();
+
+  meta_display_pong_for_serial (display, serial);
+}
+
 static void
 xdg_surface_destructor (struct wl_resource *resource)
 {
@@ -724,6 +752,24 @@ xdg_surface_set_transient_for (struct wl_client *client,
   meta_window_set_transient_for (surface->window, transient_for);
 }
 
+static void
+xdg_surface_set_margin (struct wl_client *client,
+                        struct wl_resource *resource,
+                        int32_t left_margin,
+                        int32_t right_margin,
+                        int32_t top_margin,
+                        int32_t bottom_margin)
+{
+  MetaWaylandSurfaceExtension *xdg_surface = wl_resource_get_user_data (resource);
+  MetaWaylandSurface *surface = wl_container_of (xdg_surface, surface, xdg_surface);
+
+  surface->pending.frame_extents_changed = TRUE;
+  surface->pending.frame_extents.left = left_margin;
+  surface->pending.frame_extents.right = right_margin;
+  surface->pending.frame_extents.top = top_margin;
+  surface->pending.frame_extents.bottom = bottom_margin;
+}
+
 static void
 xdg_surface_set_title (struct wl_client *client,
                        struct wl_resource *resource,
@@ -746,17 +792,6 @@ xdg_surface_set_app_id (struct wl_client *client,
   meta_window_set_wm_class (surface->window, app_id, app_id);
 }
 
-static void
-xdg_surface_pong (struct wl_client *client,
-                  struct wl_resource *resource,
-                  guint32 serial)
-{
-  MetaWaylandSurfaceExtension *xdg_surface = wl_resource_get_user_data (resource);
-  MetaWaylandSurface *surface = wl_container_of (xdg_surface, surface, xdg_surface);
-
-  meta_window_pong (surface->window, serial);
-}
-
 static gboolean
 begin_grab_op_on_surface (MetaWaylandSurface *surface,
                           MetaWaylandSeat    *seat,
@@ -792,7 +827,7 @@ xdg_surface_move (struct wl_client *client,
 
   if (seat->pointer.button_count == 0 ||
       seat->pointer.grab_serial != serial ||
-      seat->pointer.focus != surface)
+      seat->pointer.focus_surface != surface)
     return;
 
   begin_grab_op_on_surface (surface, seat, META_GRAB_OP_MOVING);
@@ -838,7 +873,7 @@ xdg_surface_resize (struct wl_client *client,
 
   if (seat->pointer.button_count == 0 ||
       seat->pointer.grab_serial != serial ||
-      seat->pointer.focus != surface)
+      seat->pointer.focus_surface != surface)
     return;
 
   begin_grab_op_on_surface (surface, seat, grab_op_for_edge (edges));
@@ -859,7 +894,8 @@ xdg_surface_set_fullscreen (struct wl_client *client,
   MetaWaylandSurfaceExtension *xdg_surface = wl_resource_get_user_data (resource);
   MetaWaylandSurface *surface = wl_container_of (xdg_surface, surface, xdg_surface);
 
-  meta_window_make_fullscreen (surface->window);
+  surface->pending.fullscreen.changed = TRUE;
+  surface->pending.fullscreen.value = TRUE;
 }
 
 static void
@@ -869,7 +905,8 @@ xdg_surface_unset_fullscreen (struct wl_client *client,
   MetaWaylandSurfaceExtension *xdg_surface = wl_resource_get_user_data (resource);
   MetaWaylandSurface *surface = wl_container_of (xdg_surface, surface, xdg_surface);
 
-  meta_window_unmake_fullscreen (surface->window);
+  surface->pending.fullscreen.changed = TRUE;
+  surface->pending.fullscreen.value = FALSE;
 }
 
 static void
@@ -879,7 +916,8 @@ xdg_surface_set_maximized (struct wl_client *client,
   MetaWaylandSurfaceExtension *xdg_surface = wl_resource_get_user_data (resource);
   MetaWaylandSurface *surface = wl_container_of (xdg_surface, surface, xdg_surface);
 
-  meta_window_maximize (surface->window, META_MAXIMIZE_HORIZONTAL | META_MAXIMIZE_VERTICAL);
+  surface->pending.maximized.changed = TRUE;
+  surface->pending.maximized.value = TRUE;
 }
 
 static void
@@ -889,7 +927,8 @@ xdg_surface_unset_maximized (struct wl_client *client,
   MetaWaylandSurfaceExtension *xdg_surface = wl_resource_get_user_data (resource);
   MetaWaylandSurface *surface = wl_container_of (xdg_surface, surface, xdg_surface);
 
-  meta_window_unmaximize (surface->window, META_MAXIMIZE_HORIZONTAL | META_MAXIMIZE_VERTICAL);
+  surface->pending.maximized.changed = TRUE;
+  surface->pending.maximized.value = FALSE;
 }
 
 static void
@@ -905,9 +944,9 @@ xdg_surface_set_minimized (struct wl_client *client,
 static const struct xdg_surface_interface meta_wayland_xdg_surface_interface = {
   xdg_surface_destroy,
   xdg_surface_set_transient_for,
+  xdg_surface_set_margin,
   xdg_surface_set_title,
   xdg_surface_set_app_id,
-  xdg_surface_pong,
   xdg_surface_move,
   xdg_surface_resize,
   xdg_surface_set_output,
@@ -919,19 +958,10 @@ static const struct xdg_surface_interface meta_wayland_xdg_surface_interface = {
 };
 
 static void
-use_unstable_version (struct wl_client *client,
-                      struct wl_resource *resource,
-                      int32_t version)
-{
-  if (version != META_XDG_SHELL_VERSION)
-    g_warning ("Bad xdg_shell version: %d", version);
-}
-
-static void
-get_xdg_surface (struct wl_client *client,
-                 struct wl_resource *resource,
-                 guint32 id,
-                 struct wl_resource *surface_resource)
+xdg_shell_get_xdg_surface (struct wl_client *client,
+                           struct wl_resource *resource,
+                           guint32 id,
+                           struct wl_resource *surface_resource)
 {
   MetaWaylandSurface *surface = wl_resource_get_user_data (surface_resource);
 
@@ -967,44 +997,29 @@ xdg_popup_destroy (struct wl_client *client,
   wl_resource_destroy (resource);
 }
 
-static void
-xdg_popup_pong (struct wl_client *client,
-                struct wl_resource *resource,
-                uint32_t serial)
-{
-  MetaWaylandSurfaceExtension *xdg_popup = wl_resource_get_user_data (resource);
-  MetaWaylandSurface *surface = wl_container_of (xdg_popup, surface, xdg_popup);
-
-  meta_window_pong (surface->window, serial);
-}
-
 static const struct xdg_popup_interface meta_wayland_xdg_popup_interface = {
   xdg_popup_destroy,
-  xdg_popup_pong,
 };
 
 static void
-get_xdg_popup (struct wl_client *client,
-               struct wl_resource *resource,
-               uint32_t id,
-               struct wl_resource *surface_resource,
-               struct wl_resource *parent_resource,
-               struct wl_resource *seat_resource,
-               uint32_t serial,
-               int32_t x,
-               int32_t y,
-               uint32_t flags)
+xdg_shell_get_xdg_popup (struct wl_client *client,
+                         struct wl_resource *resource,
+                         uint32_t id,
+                         struct wl_resource *surface_resource,
+                         struct wl_resource *parent_resource,
+                         struct wl_resource *seat_resource,
+                         uint32_t serial,
+                         int32_t x,
+                         int32_t y,
+                         uint32_t flags)
 {
   MetaWaylandSurface *surface = wl_resource_get_user_data (surface_resource);
   MetaWaylandSurface *parent_surf = wl_resource_get_user_data (parent_resource);
   MetaWaylandSeat *seat = wl_resource_get_user_data (seat_resource);
-  MetaRectangle parent_rect;
 
   if (parent_surf == NULL || parent_surf->window == NULL)
     return;
 
-  meta_window_get_frame_rect (parent_surf->window, &parent_rect);
-
   if (!create_surface_extension (&surface->xdg_popup, client, surface_resource, resource, id,
                                  META_XDG_POPUP_VERSION,
                                  &xdg_popup_interface,
@@ -1018,8 +1033,8 @@ get_xdg_popup (struct wl_client *client,
     }
 
   surface->window = meta_window_wayland_new (meta_get_display (), surface);
-  surface->window->rect.x = parent_rect.x + x;
-  surface->window->rect.y = parent_rect.y + y;
+  surface->window->rect.x = parent_surf->window->rect.x + x;
+  surface->window->rect.y = parent_surf->window->rect.y + y;
   surface->window->showing_for_first_time = FALSE;
   surface->window->placed = TRUE;
   meta_window_set_transient_for (surface->window, parent_surf->window);
@@ -1031,11 +1046,40 @@ get_xdg_popup (struct wl_client *client,
 }
 
 static const struct xdg_shell_interface meta_wayland_xdg_shell_interface = {
-  use_unstable_version,
-  get_xdg_surface,
-  get_xdg_popup,
+  xdg_shell_use_unstable_version,
+  xdg_shell_get_xdg_surface,
+  xdg_shell_get_xdg_popup,
+  xdg_shell_pong,
 };
 
+typedef struct {
+  struct wl_resource *resource;
+  struct wl_listener client_destroy_listener;
+} XdgShell;
+
+static void
+xdg_shell_handle_client_destroy (struct wl_listener *listener, void *data)
+{
+  XdgShell *xdg_shell = wl_container_of (listener, xdg_shell, client_destroy_listener);
+  g_slice_free (XdgShell, xdg_shell);
+}
+
+static struct wl_resource *
+get_xdg_shell_for_client (struct wl_client *client)
+{
+  struct wl_listener *listener;
+  XdgShell *xdg_shell;
+
+  listener = wl_client_get_destroy_listener (client, xdg_shell_handle_client_destroy);
+
+  /* No xdg_shell has been bound for this client */
+  if (listener == NULL)
+    return NULL;
+
+  xdg_shell = wl_container_of (listener, xdg_shell, client_destroy_listener);
+  return xdg_shell->resource;
+}
+
 static void
 bind_xdg_shell (struct wl_client *client,
                 void *data,
@@ -1043,10 +1087,23 @@ bind_xdg_shell (struct wl_client *client,
                 guint32 id)
 {
   struct wl_resource *resource;
+  XdgShell *xdg_shell;
 
-  resource = wl_resource_create (client, &xdg_shell_interface,
-				 MIN (META_XDG_SHELL_VERSION, version), id);
+  if (version != 1)
+    {
+      g_warning ("using xdg-shell without stable version 1\n");
+      return;
+    }
+
+  xdg_shell = g_slice_new (XdgShell);
+
+  resource = wl_resource_create (client, &xdg_shell_interface, 1, id);
   wl_resource_set_implementation (resource, &meta_wayland_xdg_shell_interface, data, NULL);
+  xdg_shell->resource = wl_resource_create (client, &xdg_shell_interface, 1, id);
+  wl_resource_set_implementation (xdg_shell->resource, &meta_wayland_xdg_shell_interface, data, NULL);
+
+  xdg_shell->client_destroy_listener.notify = xdg_shell_handle_client_destroy;
+  wl_client_add_destroy_listener (client, &xdg_shell->client_destroy_listener);
 }
 
 static void
@@ -1071,6 +1128,13 @@ set_dbus_properties (struct wl_client   *client,
   MetaWaylandSurfaceExtension *gtk_surface = wl_resource_get_user_data (resource);
   MetaWaylandSurface *surface = wl_container_of (gtk_surface, surface, gtk_surface);
 
+  /* Broken client, let it die instead of us */
+  if (!surface->window)
+    {
+      meta_warning ("meta-wayland-surface: set_dbus_properties called with invalid window!\n");
+      return;
+    }
+
   meta_window_set_gtk_dbus_properties (surface->window,
                                        application_id,
                                        unique_bus_name,
@@ -1417,8 +1481,7 @@ void
 meta_wayland_init_shell (MetaWaylandCompositor *compositor)
 {
   if (wl_global_create (compositor->wayland_display,
-			&xdg_shell_interface,
-			META_XDG_SHELL_VERSION,
+			&xdg_shell_interface, 1,
 			compositor, bind_xdg_shell) == NULL)
     g_error ("Failed to register a global xdg-shell object");
 
@@ -1438,34 +1501,46 @@ meta_wayland_init_shell (MetaWaylandCompositor *compositor)
 void
 meta_wayland_surface_configure_notify (MetaWaylandSurface *surface,
 				       int                 new_width,
-				       int                 new_height,
-				       int                 edges)
+				       int                 new_height)
 {
   if (surface->xdg_surface.resource)
     xdg_surface_send_configure (surface->xdg_surface.resource,
-                                edges, new_width, new_height);
+                                new_width, new_height);
 }
 
 void
-meta_wayland_surface_focused_set (MetaWaylandSurface *surface)
+meta_wayland_surface_activated (MetaWaylandSurface *surface)
 {
   if (surface->xdg_surface.resource)
-    xdg_surface_send_focused_set (surface->xdg_surface.resource);
+    xdg_surface_send_activated (surface->xdg_surface.resource);
 }
 
 void
-meta_wayland_surface_focused_unset (MetaWaylandSurface *surface)
+meta_wayland_surface_deactivated (MetaWaylandSurface *surface)
 {
   if (surface->xdg_surface.resource)
-    xdg_surface_send_focused_unset (surface->xdg_surface.resource);
+    xdg_surface_send_deactivated (surface->xdg_surface.resource);
 }
 
 void
 meta_wayland_surface_ping (MetaWaylandSurface *surface,
-                           guint32             timestamp)
+                           guint32             serial)
+{
+  struct wl_client *client = wl_resource_get_client (surface->resource);
+  struct wl_resource *xdg_shell = get_xdg_shell_for_client (client);
+
+  if (xdg_shell == NULL)
+    {
+      g_warning ("Trying to ping a surface without an xdg_shell bound. How does this happen?");
+      return;
+    }
+
+  xdg_shell_send_ping (xdg_shell, serial);
+}
+
+void
+meta_wayland_surface_delete (MetaWaylandSurface *surface)
 {
   if (surface->xdg_surface.resource)
-    xdg_surface_send_ping (surface->xdg_surface.resource, timestamp);
-  else if (surface->xdg_popup.resource)
-    xdg_popup_send_ping (surface->xdg_popup.resource, timestamp);
+    xdg_surface_send_delete (surface->xdg_surface.resource);
 }
